#' RainFARM Stochastic Precipitation Downscaling
#'
#' @author Jost von Hardenberg, ISAC-CNR, \email{j.vonhardenberg@isac.cnr.it}
#'
#' @description Perform RainFARM downscaling.
#' These functions implement the RainFARM stochastic precipitation downscaling method.
#' Adapted for climate downscaling and including orographic correction.
#' References: D'Onofrio et al. 2014, J of Hydrometeorology 15, 830-843; Rebora et. al 2006, JHM 7, 724
#' The functions use julia_setup from the JuliaCall library, 
#' which takes a long time only the first time it is called on a machine
#' 
#' @param pr the precipitation array to downscal
#' @param lon vector or array of longitudes
#' @param lat vector or array of latitudes
#' @param nf refinement factor for downscaling (the output resolution is increased by this factor)
#' @param slope prescribed or automatic spectral slope (default 0. == automatic)
#' @param kmin first wavenumber for automatic spectral slope
#' @param weights matrix (can be obtained using the \code{rfweights} function (default 1. == none)
#' @param nens number of ensemble members to produce
#' @param fglob logical to conseve global precipitation over the domain (default FALSE)
#' @param fsmooth logical to conserve precipitation with a smnoothing kernel (default TRUE)
#' @param verbose verbose output of the Julia package
#'
#' @return A list containing the fine-scale longitudes, latitudes and the sequence of \code{nens} downscaled fields
#'  
#' @import JuliaCall
#' @examples
#' 
#' nf <- 8   # Choose a downscaling by factor 8
#' nens <- 3 # Number of ensemble members
#'
#' # create a test array with dimension 8x8 and 20 timesteps
#' # or provide your own read from a netcdf file
#'
#' pr <- rnorm(8*8*20)
#' dim(pr) <- c(8,8,20)
#' lon_mat <- seq(10,13.5,0.5) # could also be a 2d matrix
#' lat_mat <- seq(40,43.5,0.5)
#'
#' # Create a test array of weights
#' ww <- array(1., dim = c(8 * nf, 8 * nf))
#' 
#' # ... or create proper weights using an external fine-scale climatology file
#' #     Specify a weightsfn filename if you wish to save the weights
#' 
#' ww <- rfweights("./worldclim.nc", nf, lon=lon_mat, lat=lat_mat, weightsfn="", fsmooth = TRUE)
#' 
#' # downscale using weights (ww=1. means do not use weights)
#' r <- rainfarm(pr, lon_mat, lat_mat, nf, fsmooth = TRUE, fglob = FALSE, weights = ww, nens = 2, verbose = TRUE)
#' lon_f <- r$lon
#' lat_f <- r$lat
#' rd1 <- r$data[[1]]
#' rd2 <- r$data[[2]]
#' @export

rainfarm <- function(pr, lon, lat, nf, fglob = FALSE,
                     fsmooth = TRUE, weights = 1., nens = 1,
                     verbose = FALSE, slope = 0, kmin=1) {
#  library(JuliaCall)
  julia_setup(useRCall = FALSE)
  julia_library("RainFARM")

  r <- julia_call("lon_lat_fine", lon, lat, as.integer(nf),
                  need_return = "R")
  lon_f <- r[[1]]
  lat_f <- r[[2]]

  if (slope == 0) {
    r <- julia_call("fft3d", pr, need_return = "R")
    fxp <- r[[1]]
    sx <- julia_call("fitslopex", fxp, kmin = as.integer(kmin),
                     need_return = "R")
  } else {
    sx <- slope
  }

  rd <- list()
  for (j in 1:nens) {
    rd <- c(rd, list(julia_call("rainfarm", pr, sx, as.integer(nf), weights,
                                fglob = fglob, fsmooth = fsmooth,
                                verbose = verbose, need_return = "R")))
  }
  return(list(lon = lon_f, lat = lat_f, data = rd))   
}
